<?php

// Generates a new relational database from JSON data contained in programs.json and incentives.json,
// including only the required data on categories, programs, and incentives,
// in form of PHP arrays.

// This file is executed automatically by ZanoxEngine_init.php.

// The entire database is divided into four distinct PHP arrays as follows (here: essential functions):

// $CatID_from_CatName[]:  Parameter: Category name  ->  Output: Category ID
// $ProgIDs_from_CatID[]:  Parameter: Category ID  ->  Output: Array containing the matching programm IDs
// $ProgInfos_from_ProgID[]:  Parameter: Program ID  ->  Output: Array containing the corresponding program infos
// $Incentives_from_ProgID[]:  Parameter: Program ID  ->  Output: Array containing the respective coupon data

// The database can be used from within any PHP file after being loaded as follows:

/*

include 'ZanoxEngine.php';

*/


// Read all data from programs.json into $programs_array[]:

$programs_array = json_decode(deBOM(file_get_contents('programs.json')), true);


// Loop through all programs in $programs_array[] for program infos and category data:

$CatID_from_CatName = array();
$ProgIDs_from_CatID = array();
$ProgInfos_from_ProgID = array();
$Incentives_from_ProgID = array();

foreach ($programs_array['programItems']['programItem'] as $program)  // Loop through all programs for program infos and categories...
   {
    $ProgramID = $program['@id'];
 // u8h() will handle UTF-8 multibyte characters; deCDATA() will remove CDATA tags...
    $ProgInfos_from_ProgID[$ProgramID]['name'] = u8h($program['name']);
    $ProgInfos_from_ProgID[$ProgramID]['description'] = u8h($program['description']);
    $ProgInfos_from_ProgID[$ProgramID]['url'] = $program['url'];
    $ProgInfos_from_ProgID[$ProgramID]['image'] = $program['image'];

 // $ProgInfos_from_ProgID[$ProgramID]['descriptionLocal'] = u8h(deCDATA($program['descriptionLocal']));  // Further fields can be adopted here...

    $Incentives_from_ProgID[$ProgramID] = array();  // Add program ID as key to $Incentives_from_ProgID[]

    if (!is_array($program['categories']['0']['category'])) continue;  // All category data missing!

    $categories = $program['categories']['0']['category'];

    reset($categories);
    if (!is_array($categories[key($categories)]))  // Only one category is specified
       {
        $CatID_from_CatName[u8h($categories['$'])] = $categories['@id'];
        $ProgIDs_from_CatID[$categories['@id']]['name'] = u8h($categories['$']);
        $ProgIDs_from_CatID[$categories['@id']]['Programs'][] = $ProgramID;
       }
    else  // Several categories are specified
       {
        foreach ($categories as $category)
           {
            $CatID_from_CatName[u8h($category['$'])] = $category['@id'];
            $ProgIDs_from_CatID[$category['@id']]['name'] = u8h($category['$']);
            $ProgIDs_from_CatID[$category['@id']]['Programs'][] = $ProgramID;
           }
       }
   }

$programs_array = NULL;

// Sort array $CatID_from_CatName[] alphabetically in ascending order by category name:
uksort($CatID_from_CatName, 'strcasecmp_uh');

// Save array $CatID_from_CatName[] in file CatID_from_CatName.dat:
file_put_contents('CatID_from_CatName.dat', serialize($CatID_from_CatName));
$CatID_from_CatName = NULL;

// Sort 'Programs' sub-arrays in array $ProgIDs_from_CatID[] alphabetically in ascending order by company name:
function namecomp($id1, $id2)  // Used for usort() and uksort() by company name retrieved from $ProgInfos_from_ProgID[]
   {
    global $ProgInfos_from_ProgID;
    if (isset($ProgInfos_from_ProgID[$id1]) && isset($ProgInfos_from_ProgID[$id2]))
       {
        return strcasecmp_uh($ProgInfos_from_ProgID[$id1]['name'], $ProgInfos_from_ProgID[$id2]['name']);
       }
    else
       {
        return 0;
       }
   }
foreach ($ProgIDs_from_CatID as $CatID => $progs)   // Direct sorting by reference would not work under PHP 4
   {
    usort($progs['Programs'], 'namecomp');
    $ProgIDs_from_CatID[$CatID]['Programs'] = $progs['Programs'];
   }

// Sort array $ProgIDs_from_CatID[] alphabetically in ascending order by category name:
function snamecomp($id1, $id2) { return strcasecmp_uh($id1['name'], $id2['name']); }  // Used for uasort() (here as well as further below):
uasort($ProgIDs_from_CatID, 'snamecomp');

// Save array $ProgIDs_from_CatID[] in file ProgIDs_from_CatID.dat:
file_put_contents('ProgIDs_from_CatID.dat', serialize($ProgIDs_from_CatID));
$ProgIDs_from_CatID = NULL;


// Read all data from incentives.json into $incentives_array[]:

$incentives_array = json_decode(deBOM(file_get_contents('incentives.json')), true);


// Loop through all incentives in $incentives_array[] for coupon incentives' data:

foreach ($incentives_array['incentiveItems']['incentiveItem'] as $incentive)  // Loop through all incentives for coupon data...
   {
    if ($incentive['incentiveType']!='coupons') continue;  // Incentive is not a coupon, therefore jump to next! - Comment out to use ALL types of incentives.

    $ProgramID = $incentive['program']['@id'];
 // u8h() will handle UTF-8 multibyte characters...
    if (!isset($ProgInfos_from_ProgID[$ProgramID]))  // If ID key not already present in that list, add a new record
       {
        if (isset($incentive['program']['$']))
           {
            $ProgInfos_from_ProgID[$ProgramID]['name'] = u8h($incentive['program']['$']);
           }
        else
           {
            $ProgInfos_from_ProgID[$ProgramID]['name'] = '';
           }
        $ProgInfos_from_ProgID[$ProgramID]['description'] = '';
        $ProgInfos_from_ProgID[$ProgramID]['url'] = '';
        $ProgInfos_from_ProgID[$ProgramID]['image'] = '';
       }

    $idx = 0;
    if (isset($Incentives_from_ProgID[$ProgramID])) $idx = count($Incentives_from_ProgID[$ProgramID]);  // isset() just in case...

    $Incentives_from_ProgID[$ProgramID][$idx]['id'] = $incentive['@id'];
    $Incentives_from_ProgID[$ProgramID][$idx]['name'] = u8h($incentive['name']);
    $Incentives_from_ProgID[$ProgramID][$idx]['startDate'] = fmtDate($incentive['startDate'], $DateFormat);
    $Incentives_from_ProgID[$ProgramID][$idx]['startTime'] = fmtTime($incentive['startDate'], $TimeFormat);
    if (isset($incentive['endDate']))
       {
        $Incentives_from_ProgID[$ProgramID][$idx]['endDate'] = fmtDate($incentive['endDate'], $DateFormat);  // Key 'endDate' exists...
        $Incentives_from_ProgID[$ProgramID][$idx]['endTime'] = fmtTime($incentive['endDate'], $TimeFormat);
       }
    else
       {
        $Incentives_from_ProgID[$ProgramID][$idx]['endDate'] = '';  // Key 'endDate' is MISSING...
        $Incentives_from_ProgID[$ProgramID][$idx]['endTime'] = '';
       }
    if (isset($incentive['couponCode']))
       {
        $Incentives_from_ProgID[$ProgramID][$idx]['couponCode'] = u8h($incentive['couponCode']);  // Key 'couponCode' exists
       }
    else
       {
        $Incentives_from_ProgID[$ProgramID][$idx]['couponCode'] = '';  // Key 'couponCode' is MISSING
       }
    if ($incentive['newCustomerOnly']=='1')  // Ensuring that 'newCustomerOnly' will be TRUE or FALSE...
       {
        $Incentives_from_ProgID[$ProgramID][$idx]['newCustomerOnly'] = true;
       }
    else
       {
        $Incentives_from_ProgID[$ProgramID][$idx]['newCustomerOnly'] = false;
       }

 // $Incentives_from_ProgID[$ProgramID][$idx]['...'] = u8h($incentive['...']);  // Further fields can be adopted here...

    $incentive_tr = $incentive['admedia']['admediumItem']['trackingLinks'];

    if (is_array($incentive_tr))  // Read coupon URL and image URL only if corresp. sub-array exists, otherwise type mismatch error
       {
        $Incentives_from_ProgID[$ProgramID][$idx]['url'] = $incentive_tr['trackingLink'][0]['ppc'];
        $Incentives_from_ProgID[$ProgramID][$idx]['image'] = $incentive_tr['trackingLink'][0]['ppv'];
       }
    else
       {
        $Incentives_from_ProgID[$ProgramID][$idx]['url'] = '';
        $Incentives_from_ProgID[$ProgramID][$idx]['image'] = '';
       }
   }

$incentives_array = NULL;

// Sort array $ProgInfos_from_ProgID[] alphabetically in ascending order by company name:
uasort($ProgInfos_from_ProgID, 'snamecomp');

// Save array $ProgInfos_from_ProgID[] in file ProgInfos_from_ProgID.dat:
file_put_contents('ProgInfos_from_ProgID.dat', serialize($ProgInfos_from_ProgID));

// Sort array $Incentives_from_ProgID[] alphabetically in ascending order by company name:
uksort($Incentives_from_ProgID, 'namecomp');

$ProgInfos_from_ProgID = NULL;

// Save array $Incentives_from_ProgID[] in file Incentives_from_ProgID.dat:
file_put_contents('Incentives_from_ProgID.dat', serialize($Incentives_from_ProgID));
$Incentives_from_ProgID = NULL;


print("- Processing finished! -<br>\r\n");

?>
